import numpy as np
from qtpy.QtWidgets import QDialog, QLabel, QGridLayout, QDialogButtonBox, QPushButton, QLineEdit
from qtpy.QtCore import Qt
from gcoreutils.dataset import DataSet
from gcoreutils.nsarray import NSArray
from qtdraw.core.group_view import GroupView
from qtdraw.multipie.plot_object import (
    check_get_site,
    parse_modulation_list,
    create_modulated_samb_object,
    plot_modulated_vector_cluster,
    plot_modulated_orbital_cluster,
)


# ==================================================
class DialogModulation(QDialog):
    # ==================================================
    def __init__(self, basis, modulation, head, is_orbital=False, parent=None):
        """
        initialize the class.

        Args:
            basis (list): valid basis list
            modulation (str): modulation list.
            head (str): multipole type.
            is_orbital (bool, optional): orbital plot ?
            parent (QWidget, optional): parent object (DialogGroup).
        """
        super().__init__(parent)
        title = "Modulation"
        if is_orbital:
            title += " - " + "orbital"
        else:
            title += " - " + "vector"
        self.setWindowTitle(title)

        self.parent = parent
        self.basis = basis
        self.head = head
        self.is_orbital = is_orbital
        self.org_range = parent._qtdraw.setting["view_range"]
        if is_orbital:
            self.name = "Z_" + self.parent._qtdraw._get_name("orbital")
        else:
            self.name = "Z_" + self.parent._qtdraw._get_name("vector")

        # set modulation data.
        data = parse_modulation_list(modulation)
        if data is None:
            data = [[]]
        data = [[str(no), basis.index(i[1]), i[0], i[2], i[3]] for no, i in enumerate(data)]
        self.count = len(data)

        role = [
            ("hide", "0"),
            ("combo", basis, 0),
            ("math", "s_scalar", "1"),
            ("math", "s_vector", "[0,0,0]"),
            ("combo", ["cos", "sin"], 0),
        ]
        form = {"list": [["No", "basis", "coeff", "k", "phase"], role, data]}
        self.modulation_data = DataSet(form)

        # modulation panel.
        self.modulation = GroupView(self.modulation_data, "list", parent=parent)
        self.modulation.add_button.hide()
        self.modulation.insert_button.hide()
        self.modulation.remove_button.hide()

        # range.
        self.offset_label = QLabel("lower", self)
        self.offset = QLineEdit("[0,0,0]", self)
        self.repeat_label = QLabel("repeat", self)
        self.repeat = QLineEdit("[1,1,1]", self)

        # buttons.
        self.add_button = QPushButton("add", self)
        self.add_button.setFocusPolicy(Qt.NoFocus)
        self.remove_button = QPushButton("remove", self)
        self.remove_button.setFocusPolicy(Qt.NoFocus)

        self.buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel | QDialogButtonBox.Apply)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)
        self.buttonBox.clicked.connect(self.apply)

        # main layout.
        self.resize(512, 400)
        self.layout = QGridLayout(self)
        self.layout.addWidget(self.modulation, 0, 0, 8, 4)
        self.layout.addWidget(self.add_button, 8, 0, 1, 2)
        self.layout.addWidget(self.remove_button, 8, 2, 1, 2)

        self.layout.addWidget(self.offset_label, 9, 0, 1, 1)
        self.layout.addWidget(self.offset, 9, 1, 1, 1)
        self.layout.addWidget(self.repeat_label, 9, 2, 1, 1)
        self.layout.addWidget(self.repeat, 9, 3, 1, 1)

        self.layout.addWidget(self.buttonBox, 10, 1, 1, 3)

        # connections.
        self.add_button.clicked.connect(self.add_data)
        self.remove_button.clicked.connect(self.remove_data)
        self.finished.connect(self.close)

    # ==================================================
    def add_data(self):
        self.modulation.model.appendRow(name=str(self.count))
        self.count += 1

    # ==================================================
    def remove_data(self):
        index = self.modulation.view.selectedIndexes()
        if len(index) != 0:
            self.modulation.model.removeRow(index[0])

    # ==================================================
    def accept(self):
        self.remove_plot_data(self.name)
        self.plot_modulation()
        self.parent._qtdraw._garbage_collection()
        data_str = self.get_raw_data()[1]
        if data_str is not None:
            if self.is_orbital:
                self.parent.tab2_orbital_modulation_list.setText(data_str)
            else:
                self.parent.tab2_vector_modulation_list.setText(data_str)
        super().accept()

    # ==================================================
    def reject(self):
        self.parent._qtdraw.set_range(self.org_range)
        self.remove_plot_data(self.name)
        self.parent._qtdraw._garbage_collection()
        super().reject()

    # ==================================================
    def apply(self, button):
        if self.buttonBox.standardButton(button) == QDialogButtonBox.Apply:
            self.remove_plot_data(self.name)
            self.parent._qtdraw._garbage_collection()
            self.plot_modulation()

    # ==================================================
    def get_raw_data(self):
        """
        get raw modulation list.

        Returns: tuple.
            - list: modulation list.
            - str: modulation list in str.
        """
        data = self.modulation_data.to_data()["list"][2]
        data = [[self.basis[i[1]]] + i[2:] for i in data if i[0] != ""]
        if len(data) < 1:
            return None, None
        data_str = str([i[:-1] + ["cos"] if i[-1] == 0 else i[:-1] + ["sin"] for i in data]).replace("'", "")

        return data, data_str

    # ==================================================
    def plot_modulation(self):
        """
        plot modulation.
        """
        rng_lu, rng = self.get_range()
        if rng_lu is None:
            return
        data = self.get_raw_data()[0]
        if data is None:
            return

        self.parent._qtdraw.set_range(rng_lu)

        v = NSArray.vector3d()
        if self.is_orbital:
            site = self.parent.tab2_orbital_proj_site
            obj, igrid = create_modulated_samb_object(
                self.parent.tab2_orbital_proj_z_samb,
                site,
                self.parent.tab2_orbital_proj_c_samb,
                self.parent._pgroup,
                v,
                self.head,
                data,
                rng[1],
                rng[0],
                self.parent.pset,
            )

            plot_modulated_orbital_cluster(self.parent._qtdraw, site, obj, self.name, self.parent.pset, igrid, self.head)

        else:
            site = self.parent.tab2_vector_proj_site
            obj, igrid = create_modulated_samb_object(
                self.parent.tab2_vector_proj_z_samb,
                site,
                self.parent.tab2_vector_proj_c_samb,
                self.parent._pgroup,
                v,
                self.head,
                data,
                rng[1],
                rng[0],
                self.parent.pset,
            )

            plot_modulated_vector_cluster(self.parent._qtdraw, site, obj, self.name, self.parent.pset, igrid, self.head, v)

    # ==================================================
    def get_range(self):
        """
        get view range.

        Returns: tuple.
            - list: offset, upper.
            - list: offset, repeat.
        """
        offset = check_get_site(self.offset.text())
        if offset is None:
            return None
        else:
            offset = offset.numpy().astype(int).tolist()
        repeat = check_get_site(self.repeat.text())
        if repeat is None:
            return None
        else:
            repeat = repeat.numpy().astype(int).tolist()

        upper = (np.array(offset) + np.array(repeat)).tolist()
        return [offset, upper], [offset, repeat]

    # ==================================================
    def remove_plot_data(self, name):
        """
        remove plot data from qtdraw.

        Args:
            name (str): vector or orbital.
        """
        group = "orbital" if self.is_orbital else "vector"
        df = self.parent._qtdraw.dataset[group]
        if len(df) > 0:
            df0 = df[df["name"] == name].copy().reset_index(drop=True)
            for i in range(len(df0)):
                obj_id = df0.at[i, "AD"]
                lbl_id = df0.at[i, "AL"]
                if obj_id != "":
                    self.parent._qtdraw._remove_actor(obj_id)
                if lbl_id != "":
                    self.parent._qtdraw._remove_actor(lbl_id)
                self.parent._qtdraw.dataset[group].at[i, "name"] = ""

    # ==================================================
    def keyPressEvent(self, event):
        """
        to prevent enter input.
        """
        return

    # ==================================================
    def close(self):
        if self.is_orbital:
            self.parent.tab2_modulated_orbital_active = False
        else:
            self.parent.tab2_modulated_vector_active = False
        super().close()



preference = {
    # light.
    "light.eye_dome_lighting": False,
    "light.pbr": False,
    "light.metallic": 0.1,
    "light.roughness": 0.2,
    "light.intensity": 0.5,
    # label.
    "label.font": "arial",  # "courier/times/arial"
    "label.size": 18,
    "label.bold": False,
    "label.italic": False,
    "label.color": "licorice",
    # vector.
    "vector.tip.length": 0.35,
    "vector.tip.radius": 0.04,
    "vector.shaft.radius": 0.02,
    # cell.
    "cell.width": 1.0,
    "cell.color": "black",
    "cell.opacity": 1.0,
    # spotlight.
    "spotlight.color": "orange",
    "spotlight.opacity": 0.3,
    "spotlight.size": 0.15,
    # axis.
    "axis.type": "xyz",
    "axis.size": 18,
    "axis.bold": True,
    "axis.italic": False,
    "axis.position": 1.0,
}

rcParams = {
    # default of initial setting.
    "init.title": "QtDraw",
    "init.model": "untitled",
    "init.origin": "[0.0,0.0,0.0]",
    "init.view": "[4,5,2]",
    "init.size": "[1174,768]",  # (width, height) including side panel
    "init.view_range": ["[0.0,0.0,0.0]", "[1.01,1.01,1.01]"],
    "init.parallel_projection": True,
    "init.axis": "on",  # "on/axis/off"
    "init.cell": "single",  # "single/all/off"
    "init.grid": False,
    #
    # plotter setting.
    "plotter.ui": "qtdraw.ui",
    "plotter.style": "fusion",  # "fusion/windows/macintosh"
    "plotter.font.family": "Helvetica Neue",  # "Monaco/Osaka/Times New Roman/Arial/Helvetica Neue"
    "plotter.font.size": 13,  # pixel
    "plotter.theme": "document",  # "document/paraview/default/dark"
    "plotter.ext": ".qtdw",
    "plotter.cif": ".cif",
    "plotter.vesta": ".vesta",
    "plotter.color_scheme": "VESTA",  # "VESTA/Jmol"
    "plotter.site_scale": 1.2,
    "plotter.bond_scale": 1.2,
    #
    # default for plotting.
    "plot.orbital.theta": 180,
    "plot.orbital.phi": 180,
    #
    # detailed setting.
    "detail.axis.shaft.radius": 0.05,
    "detail.axis.tip.radius": 0.1,
    "detail.axis.tip.length": 0.4,
    "detail.axis.tip.resolution": 60,
    "detail.axis.center.radius": 0.14,
    "detail.axis.center.theta": 60,
    "detail.axis.center.phi": 60,
    "detail.axis.viewport": (0.0, 0.0, 0.25, 0.25),
    "detail.axis.scale": 2.2,
    "detail.axis.label.position": [[0.26, -0.52, 0.0], [-0.44, 0.26, 0.0], [0.0, 0.0, 0.45]],
    "detail.axis.label": {"xyz": ("x", "y", "z"), "abc": ("a", "b", "c"), "abc*": ("a*", "b*", "c*")},
    "detail.axis.label.color": ("black", "black", "black"),
    "detail.axis.color": ["crimson", "forestgreen", "royalblue"],
    "detail.axis.color.center": "snow",
    "detail.smooth.samples": 10,
    "detail.smooth.line": True,
    "detail.smooth.point": True,
    "detail.smooth.polygon": True,
    "detail.smooth.shading": True,
    "detail.smooth.anti_aliasing": False,  # if use True, it does not work correctly for some environment
    "detail.latex.color": "black",
    "detail.latex.dpi": 300,
    "detail.latex.format": "png",  # "png/svg/jpg"
    "detail.image.width": 100,
    "detail.table.bg_color": "lightgrey",
    "detail.screenshot.ext": ".png",
    "detail.screenshot.ifile": "*.png *.bmp *.tif *.tiff",  # jpeg, jpg do not work.
    "detail.screenshot.gfile": "*.svg *.eps *.ps *.pdf",
    #
    # dataset infomation setting.
    "detail.dataset.property": {  # last column in the role is default value.
        "site": (
            ["name", "ON", "label", "cell", "position", "size", "color", "opacity", "space", "D", "AD", "AL"],
            [
                ("check", "D", "AD", "S"),
                ("check", "ON", "", False),
                ("text", "S"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                ("text", "real_positive", 2, 1.0),
                ("color", "darkseagreen"),
                ("text", "real_unit", 2, 1.0),
                ("text", "int_positive", 3),
                ("hide", True),
                ("hide", ""),
                ("hide", ""),
            ],
        ),
        "bond": (
            [
                "name",
                "ON",
                "label",
                "cell",
                "position",
                "vector",
                "width",
                "color",
                "color2",
                "opacity",
                "space",
                "D",
                "AD",
                "AL",
            ],
            [
                ("check", "D", "AD", "b"),
                ("check", "ON", "", False),
                ("text", "b"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                ("text", "r_vector", [0.0, 0.0, 1.0]),
                ("text", "real_positive", 2, 1.0),
                ("color", "silver"),
                ("color", "silver"),
                ("text", "real_unit", 2, 1.0),
                ("text", "int_positive", 3),
                ("hide", True),
                ("hide", ""),
                ("hide", ""),
            ],
        ),
        "vector": (
            [
                "name",
                "ON",
                "label",
                "cell",
                "position",
                "vector",
                "length",
                "width",
                "offset",
                "color",
                "opacity",
                "space",
                "D",
                "AD",
                "AL",
            ],
            [
                ("check", "D", "AD", "vec"),
                ("check", "ON", "", False),
                ("text", "vec"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                ("text", "r_vector", [0.0, 0.0, 1.0]),
                ("text", "real_positive", 2, 1.0),
                ("text", "real_positive", 2, 1.0),
                ("text", "real", 2, -0.43),
                ("color", "orange"),
                ("text", "real_unit", 2, 1.0),
                ("text", "int_positive", 3),
                ("hide", True),
                ("hide", ""),
                ("hide", ""),
            ],
        ),
        "orbital": (
            [
                "name",
                "ON",
                "label",
                "cell",
                "position",
                "shape",
                "surface",
                "size",
                "scale",
                "theta0",
                "theta1",
                "phi0",
                "phi1",
                "color",
                "opacity",
                "space",
                "D",
                "AD",
                "AL",
            ],
            [
                ("check", "D", "AD", "orb"),
                ("check", "ON", "", False),
                ("text", "orb"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                ("math", "s_scalar", ["x", "y", "z", "r"], "1"),
                ("math", "s_scalar", ["x", "y", "z", "r"], "0"),
                ("text", "real_positive", 2, 0.1),
                ("check", "scale", "", True),
                ("text", "int_positive", 0),
                ("text", "int_positive", 180),
                ("text", "int_positive", 0),
                ("text", "int_positive", 360),
                ("color_both", "Wistia"),
                ("text", "real_unit", 2, 1.0),
                ("text", "int_positive", 3),
                ("hide", True),
                ("hide", ""),
                ("hide", ""),
            ],
        ),
        "stream": (
            [
                "name",
                "ON",
                "label",
                "cell",
                "position",
                "shape",
                "vector",
                "size",
                "v_size",
                "width",
                "scale",
                "theta",
                "phi",
                "theta0",
                "theta1",
                "phi0",
                "phi1",
                "color",
                "component",
                "opacity",
                "space",
                "D",
                "AD",
                "AL",
            ],
            [
                ("check", "D", "AD", "stm"),
                ("check", "ON", "", False),
                ("text", "stm"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                ("math", "s_scalar", ["x", "y", "z", "r"], "1"),
                ("math", "s_column_vector", ["x", "y", "z", "r"], "[x, y, z]"),
                ("text", "real_positive", 2, 1.0),
                ("text", "real_positive", 2, 0.4),
                ("text", "real_positive", 2, 1.0),
                ("check", "scale", "", False),
                ("text", "int_positive", 20),
                ("text", "int_positive", 15),
                ("text", "int_positive", 0),
                ("text", "int_positive", 180),
                ("text", "int_positive", 0),
                ("text", "int_positive", 360),
                ("color_both", "coolwarm"),
                ("combo", ["x", "y", "z", "abs"], 3),
                ("text", "real_unit", 2, 1.0),
                ("text", "int_positive", 3),
                ("hide", True),
                ("hide", ""),
                ("hide", ""),
            ],
        ),
        "plane": (
            ["name", "ON", "label", "cell", "position", "normal", "x", "y", "color", "opacity", "space", "D", "AD", "AL"],
            [
                ("check", "D", "AD", "pl"),
                ("check", "ON", "", False),
                ("text", "pl"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                ("text", "r_vector", 2, [0.0, 0.0, 1.0]),
                ("text", "real_positive", 2, 1.0),
                ("text", "real_positive", 2, 1.0),
                ("color", "sky"),
                ("text", "real_unit", 2, 1.0),
                ("text", "int_positive", 3),
                ("hide", True),
                ("hide", ""),
                ("hide", ""),
            ],
        ),
        "box": (
            [
                "name",
                "ON",
                "label",
                "cell",
                "position",
                "a1",
                "a2",
                "a3",
                "edge",
                "wireframe",
                "width",
                "color",
                "opacity",
                "space",
                "D",
                "AD",
                "AL",
            ],
            [
                ("check", "D", "AD", "box"),
                ("check", "ON", "", False),
                ("text", "box"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                ("text", "r_vector", [1.0, 0.0, 0.0]),
                ("text", "r_vector", [0.0, 1.0, 0.0]),
                ("text", "r_vector", [0.0, 0.0, 1.0]),
                ("check", "edge", "", False),
                ("check", "wireframe", "", False),
                ("text", "real_positive", 2, 1.0),
                ("color", "honeydew"),
                ("text", "real_unit", 2, 1.0),
                ("text", "int_positive", 3),
                ("hide", True),
                ("hide", ""),
                ("hide", ""),
            ],
        ),
        "polygon": (
            [
                "name",
                "ON",
                "label",
                "cell",
                "position",
                "point",
                "connection",
                "edge",
                "wireframe",
                "width",
                "color",
                "opacity",
                "space",
                "D",
                "AD",
                "AL",
            ],
            [
                ("check", "D", "AD", "pl"),
                ("check", "ON", "", False),
                ("text", "pl"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                (
                    "text",
                    "r_vector_list",
                    [[0.0, 0.0, 0.0], [0.2, 0.0, 0.0], [0.0, 0.2, 0.0], [0.0, 0.0, 0.2]],
                ),
                ("text", "i_pos_matrix", [[0, 1, 2], [0, 1, 3], [1, 2, 3], [2, 0, 3]]),
                ("check", "edge", "", True),
                ("check", "wireframe", "", False),
                ("text", "real_positive", 2, 2.0),
                ("color", "aluminum"),
                ("text", "real_unit", 2, 0.5),
                ("text", "int_positive", 3),
                ("hide", True),
                ("hide", ""),
                ("hide", ""),
            ],
        ),
        "text3d": (
            [
                "name",
                "ON",
                "label",
                "cell",
                "position",
                "text",
                "size",
                "depth",
                "normal",
                "offset",
                "color",
                "opacity",
                "space",
                "D",
                "AD",
                "AL",
            ],
            [
                ("check", "D", "AD", "txt3"),
                ("check", "ON", "", False),
                ("text", "txt3"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                ("text", "string", "text"),
                ("text", "real_positive", 2, 1.0),
                ("text", "real_positive", 2, 1.0),
                ("text", "r_vector", 2, [0.0, 0.0, 1.0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                ("color", "iron"),
                ("text", "real_unit", 2, 1.0),
                ("text", "int_positive", 3),
                ("hide", True),
                ("hide", ""),
                ("hide", ""),
            ],
        ),
        "spline": (
            [
                "name",
                "ON",
                "label",
                "cell",
                "position",
                "point",
                "width",
                "n_interp",
                "closed",
                "natural",
                "color",
                "opacity",
                "space",
                "D",
                "AD",
                "AL",
            ],
            [
                ("check", "D", "AD", "spl"),
                ("check", "ON", "", False),
                ("text", "spl"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                ("text", "r_vector_list", [[0.0, 0.0, 0.0], [1.0, 0.0, 1.0], [0.0, 1.0, 2.0]]),
                ("text", "real_positive", 2, 1.0),
                ("text", "int_positive", 100),
                ("check", "closed", "", False),
                ("check", "natural", "", True),
                ("color", "banana"),
                ("text", "real_unit", 2, 1.0),
                ("text", "int_positive", 3),
                ("hide", True),
                ("hide", ""),
                ("hide", ""),
            ],
        ),
        "spline_t": (
            [
                "name",
                "ON",
                "label",
                "cell",
                "position",
                "expression",
                "t_range",
                "width",
                "n_interp",
                "closed",
                "natural",
                "color",
                "opacity",
                "space",
                "D",
                "AD",
                "AL",
            ],
            [
                ("check", "D", "AD", "spt"),
                ("check", "ON", "", False),
                ("text", "spt"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector", [0.0, 0.0, 0.0]),
                ("math", "s_column_vector", ["t"], "[cos(2 pi t), sin(2 pi t), t/2]"),
                ("text", "r_vector", [0.0, 2.0, 0.01]),
                ("text", "real_positive", 2, 1.0),
                ("text", "int_positive", 100),
                ("check", "closed", "", False),
                ("check", "natural", "", True),
                ("color", "banana"),
                ("text", "real_unit", 2, 1.0),
                ("text", "int_positive", 3),
                ("hide", True),
                ("hide", ""),
                ("hide", ""),
            ],
        ),
        "caption": (
            ["name", "cell", "position", "caption", "space", "size", "bold", "color", "D", "AD"],
            [
                ("check", "D", "AD", "cap"),
                ("text", "i_vector", [0, 0, 0]),
                ("text", "r_vector_list", [[0.0, 0.0, 0.0], [1.0, 0.0, 0.0]]),
                ("text", "s_column", ["0", "1"]),
                ("text", "int_positive", 0),
                ("text", "int_positive", 18),
                ("check", "bold", "", False),
                ("color", "licorice"),
                ("hide", True),
                ("hide", ""),
            ],
        ),
        "text": (
            ["name", "position", "relative", "caption", "size", "color", "font", "D", "AD"],
            [
                ("check", "D", "AD", "txt"),
                ("text", "r_list", [0.02, 0.95]),
                ("check", "relative", "", True),
                ("text", "latex", "text"),
                ("text", "int_positive", 8),
                ("color", "licorice"),
                ("combo", ["arial", "times", "courier"], 0),
                ("hide", True),
                ("hide", ""),
            ],
        ),
    },
}
